"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const fs = require("fs");
const path = require("path");
const Koa = require("koa");
const Router = require("koa-router");
const serve = require("koa-static");
const SwaggerUIDist = require("swagger-ui-dist");
const commonFlags = require("../common/flags");
const definition_1 = require("../common/definition");
const koa_1 = require("../common/koa");
class SwaggerUI extends command_1.Command {
    async run() {
        const { flags } = this.parse(SwaggerUI);
        const { definition, port } = flags;
        const app = new Koa();
        const router = new Router();
        let document = null;
        if (definition) {
            if (definition.match('://') && !flags.server) {
                // use remote definition
                document = definition;
            }
            else {
                // parse definition
                router.get('/openapi.json', async (ctx) => {
                    const def = await definition_1.parseDefinition({ definition });
                    if (flags.server) {
                        const addServers = flags.server.map((url) => ({ url }));
                        def.servers = def.servers ? [...def.servers, ...addServers] : addServers;
                    }
                    ctx.body = def;
                });
                document = '/openapi.json';
            }
        }
        const swaggerUIRoot = SwaggerUIDist.getAbsoluteFSPath();
        if (document) {
            const indexHTML = fs.readFileSync(path.join(swaggerUIRoot, 'index.html')).toString('utf8');
            router.get('/', (ctx) => {
                ctx.body = indexHTML
                    // use our openapi definition
                    .replace('https://petstore.swagger.io/v2/swagger.json', document)
                    // display operation ids
                    .replace('layout: "StandaloneLayout"', 'layout: "StandaloneLayout", displayOperationId: true');
            });
        }
        app.use(router.routes());
        app.use(serve(swaggerUIRoot));
        const { port: portRunning } = await koa_1.startServer({ app, port });
        this.log(`Swagger UI running at http://localhost:${portRunning}`);
    }
}
SwaggerUI.description = 'serve or bundle a Swagger UI instance';
SwaggerUI.examples = ['$ openapi swagger-ui', '$ openapi swagger-ui -d ./openapi.yml'];
SwaggerUI.flags = Object.assign({}, commonFlags.help(), commonFlags.definition(), commonFlags.port(), commonFlags.servers());
SwaggerUI.args = [];
exports.default = SwaggerUI;
